#     Copyright (c) 2013-2016 OSIsoft, LLC. All rights reserved.
# 
#     THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
#     OSIsoft, LLC.  USE, DISCLOSURE, OR REPRODUCTION IS PROHIBITED WITHOUT
#     THE PRIOR EXPRESS WRITTEN PERMISSION OF OSIsoft, LLC.
# 
#     RESTRICTED RIGHTS LEGEND
#     Use, duplication, or disclosure by the Government is subject to restrictions
#     as set forth in subparagraph (c)(1)(ii) of the Rights in Technical Data and
#     Computer Software clause at DFARS 252.227.7013
# 
#     OSIsoft, LLC
#     777 Davis Street, Suite 250, San Leandro CA 94577

require 'fluent/output'
require 'json'
require 'rest-client'

module Fluent
  class Pi < BufferedOutput
    Fluent::Plugin.register_output('pi', self)

    helpers :inject, :formatter, :compat_parameters

    piwebapiserver = 'localhost', tagpath = nil 
    valuetag = 'value'
    username = nil, pwd = nil, anon = false
    webid = nil

    DEFAULT_FORMAT_TYPE = 'json'

    config_section :buffer do
      config_set_default :chunk_keys, ['tag']
      config_set_default :flush_at_shutdown, true
      config_set_default :chunk_limit_size, 10 * 1024
    end

    config_section :format do
      config_set_default :@type, DEFAULT_FORMAT_TYPE
    end

    def prefer_buffered_processing
      false
    end

    def prefer_delayed_commit
      @delayed
    end

    attr_accessor :delayed

    def initialize
      super
      @delayed = false
    end

    def configure(conf)
      if conf['output_type'] && !conf['format']
        conf['format'] = conf['output_type']
      end
      compat_parameters_convert(conf, :inject, :formatter)

      super

      @formatter = formatter_create(conf: conf.elements('format').first, default_type: DEFAULT_FORMAT_TYPE)

      #set piwebapi URI information
      @piwebapiserver = conf['piwebapiserver']
      
      (conf.key?('tagpath')) ? @tagpath = conf['tagpath'] : @tagpath = nil
      (conf.key?('attpath')) ? @attpath = conf['attpath'] : @attpath = nil
       
      if(@tagpath == nil && @attpath == nil)
        $log.write("Neither a tag nor attribute is specified.  Unable to log to PI.\n")
        return
      end

      if(@tagpath != nil && @attpath != nil)
        $log.write("Both a tag and attribute is specified.  Using the attribute.\n")
      end
      
      (conf.key?('valuetag')) ? @valuetag = conf['valuetag'] : @valuetag = 'Value'
      (conf.key?('username')) ? @username = conf['username'] : @username = nil    
      (conf.key?('pwd')) ? @pwd = conf['pwd'] : @pwd = nil
      (conf.key?('anon')) ? @anon = conf['anon'] : @anon = false

      (conf.key?('webid')) ? @webid = conf['webid'] : @webid = nil
      
      if((@username == nil) && !@anon)
        $log.write("Please specify a username, or specify that anonymous will be used.\n")
        return
      end
    end

    def start
      super
      # need to populate webid if not provided

      if(@webid != nil)
        return
      end

      client = nil  
     #PI Web API search doesn't work from basic or anonymous.  Kerberos from Ruby isn't working.  I think we have to use the absolute path.  With Kerberos we could looking based on search and use top result
      if(@attpath == nil)
        url = "https://#{@piwebapiserver}/piwebapi/points/?path=#{@tagpath}"
        if(@anon)
          client = RestClient::Resource.new(url, :verify_ssl => OpenSSL::SSL::VERIFY_NONE )
        else
          client = RestClient::Resource.new(url, :verify_ssl => OpenSSL::SSL::VERIFY_NONE, :user => @username, :password =>@pwd)
        end
      else 
        url = "https://#{@piwebapiserver}/piwebapi/attributes/?path=#{@attpath}"
        if(@anon)
          client = RestClient::Resource.new(url, :verify_ssl => OpenSSL::SSL::VERIFY_NONE )
        else
          client = RestClient::Resource.new(url, :verify_ssl => OpenSSL::SSL::VERIFY_NONE, :user => @username, :password =>@pwd)
        end
      end
                                  
      response = client.get
      @webid = (JSON.parse(response.body))['WebId']
      if(@webid == nil)
        $log.write("WedID is lost.  Unable to log to PI\n")
        return
      end

      @writeURL = "https://#{@piwebapiserver}/piwebapi/streams/#{@webid}/recorded"
      if(@anon)
        @writeclient = RestClient::Resource.new(@writeURL, :verify_ssl => OpenSSL::SSL::VERIFY_NONE)
      else
        @writeclient = RestClient::Resource.new(@writeURL, :verify_ssl => OpenSSL::SSL::VERIFY_NONE,:user => @username, :password =>@pwd)
      end

    end

    def write(chunk)
        data= chunk.read
        jsonObj = "[" + data.chop + "]" # removes trailing ',' and makes a valid json object            

        response = @writeclient.post jsonObj, :content_type => 'application/json'                   
        $log.write("PI successful writes\n")
    end

    def format(tag, time, record)
        value = record[@valuetag].to_s
        "{\"Timestamp\":\"#{Time.at(time).localtime}\",\"Value\":\"#{value}\"},"
    end    
  end
end
